if [ ! -e /etc/smartd.conf ]
then
    /usr/bin/mkdir -p /etc
    /usr/bin/cp /etc/defaults/etc/smartd.conf /etc/smartd.conf
fi

if [ ! -e /etc/smartd_warning.sh ]
then
    /usr/bin/mkdir -p /etc
    /usr/bin/cp /etc/defaults/etc/smartd_warning.sh /etc/smartd_warning.sh
fi

/usr/sbin/update-smart-drivedb --install
